/*
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *  Data graphing implemented by Kileen Cheng.   Thanks for your help!
 *
*/

#include <qapplication.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_curve.h>
#include <qwt_scale_draw.h>
#include <qwt_scale_widget.h>
#include <qwt_legend.h>
#include <qwt_legend_item.h>
#include <QTime>
#include "sensorplot.h"


class TimeScaleDraw: public QwtScaleDraw
{
public:
    TimeScaleDraw(const QTime &base):
        baseTime(base)
    {
    }
	virtual QwtText label(double v) const
    {
        QTime upTime = baseTime.addSecs((int)v);
        return upTime.toString();

		//QDateTime datetime;
		//QString res;

		//datetime.setTime_t((uint)v);
		//res=QDate::shortMonthName(datetime.date().month());
		//return res;

    }
private:
    QTime baseTime;
};

class Background: public QwtPlotItem
{
public:
    Background()
    {
        setZ(0.0);
    }

    virtual int rtti() const
    {
        return QwtPlotItem::Rtti_PlotUserItem;
    }

    virtual void draw(QPainter *painter,
        const QwtScaleMap &, const QwtScaleMap &yMap,
        const QRect &rect) const
    {
        QColor c(Qt::white);
        QRect r = rect;

        for ( int i = 100; i > 0; i -= 10 )
        {
            r.setBottom(yMap.transform(i - 10));
            r.setTop(yMap.transform(i));
            painter->fillRect(r, c);

            c = c.dark(110);
        }
    }
};

class CpuCurve: public QwtPlotCurve
{
public:
    CpuCurve(const QString &title):
        QwtPlotCurve(title)
    {
#if QT_VERSION >= 0x040000
        setRenderHint(QwtPlotItem::RenderAntialiased);
#endif
    }

    void setColor(const QColor &color)
    {
#if QT_VERSION >= 0x040000
        QColor c = color;
        c.setAlpha(150);

        setPen(c);
        setBrush(c);
#else
        setPen(color);
        setBrush(QBrush(color, Qt::Dense4Pattern));
#endif
    }
};

SensorPlot::SensorPlot(QWidget *parent):
    QwtPlot(parent)
{

	currentNode = 0;
    setAutoReplot(false);

    plotLayout()->setAlignCanvasToScales(true);

    QwtLegend *legend = new QwtLegend;
    legend->setItemMode(QwtLegend::CheckableItem);
    insertLegend(legend, QwtPlot::RightLegend);

    setAxisTitle(QwtPlot::xBottom, " Time [h:m:s]");

    /*
     In situations, when there is a label at the most right position of the
     scale, additional space is needed to display the overlapping part
     of the label would be taken by reducing the width of scale and canvas.
     To avoid this "jumping canvas" effect, we add a permanent margin.
     We don't need to do the same for the left border, because there
     is enough space for the overlapping label below the left scale.
     */

    QwtScaleWidget *scaleWidget = axisWidget(QwtPlot::xBottom);
    const int fmh = QFontMetrics(scaleWidget->font()).height();
    scaleWidget->setMinBorderDist(0, fmh / 2);

    //setAxisTitle(QwtPlot::yLeft, "Temperature [F]");
    setAxisScale(QwtPlot::yLeft, -40, 105);		// scale starting by -40 degrees for celsius

    //Background *bg = new Background();
    //bg->attach(this);			// remove the background


	CpuCurve *curve;

	curve = new CpuCurve("Strength");
	curve->setColor(Qt::black);
	curve->setZ(curve->z() - 2);
	curve->attach(this);	
	data[Strength].curve = curve;

	curve = new CpuCurve("Temperature");
	curve->setColor(Qt::darkCyan);
	curve->setZ(curve->z() - 3);
	curve->attach(this);	
	data[Temperature].curve = curve;		

	showCurve(data[Strength].curve, false);
	showCurve(data[Temperature].curve, true);

	for ( int k = 0; k < HISTORY; k++ )
		timeData[HISTORY - 1 - k] = k;

	for ( int i = 0; i < MAXNODES; i++)
		for ( int j = 0; j < HISTORY; j++)
			timeDataList[i][j] = timeData[j];

	for (int j = 0; j < MAXNODES; j++)
		for (int jj = 0; jj < NCpuData; jj++)
		{
			yAxisMinList[j][jj] = 999;
			yAxisMaxList[j][jj] = -999;
		}

	for (int ii=0; ii<MAXNODES; ii++)
		dataCountList[ii] = 0;

    connect(this, SIGNAL(legendChecked(QwtPlotItem *, bool)),
        SLOT(showCurve(QwtPlotItem *, bool)));	
		
}


void SensorPlot::showCurve(QwtPlotItem *item, bool on)
{
    item->setVisible(on);
    QWidget *w = legend()->find(item);
    if ( w && w->inherits("QwtLegendItem") )
        ((QwtLegendItem *)w)->setChecked(on);

	// ??? kileen todo - Autoscaling
	//QList<QWidget *> legendGraphItem = legend()->legendItems();
	//for (int i = 0; i < legend()->itemCount(); i++)
	//	if (legendGraphItem[i] == item)
	//	{
	//		if (on)
	//		{
	//			yMin = 
	//		}
	//		break
	//	}

    replot();
}

void SensorPlot::graphData(const QString &eventData)
{
	// Get the node that has received data
	QString nodeStr = eventData.section(',', 0, 0);
	nodeStr = nodeStr.trimmed();
	nodeStr = nodeStr.at(nodeStr.length()-1);	
	int nodeNum = nodeStr.toInt();
	
	QString currentTime = eventData.section(',', 6, 6);
	currentTime = currentTime.trimmed();
	QTime qCurrTime = QTime::fromString(currentTime, "hh:mm:s");

	// Get strength info
	QString strengthStr = eventData.section(',', 3, 3);
	strengthStr = strengthStr.trimmed();
	strengthStr.resize(strengthStr.length()-1); // remove last ('%') character
	strengthStr.remove(0,9);
	int strength = strengthStr.toInt();	

	// Get temperature info
	QString degreesStr = eventData.section(',', 1, 1);
	degreesStr = degreesStr.trimmed();
	degreesStr.resize(degreesStr.length()-1); // remove last ('F') character
	degreesStr.remove(0, 5);
	double degrees = degreesStr.toDouble();
	
	
	for ( int i = dataCountList[nodeNum]; i > 0; i-- )
    {
        for ( int c = 0; c < NCpuData; c++ )
        {
            if ( i < HISTORY )
				nodeDataList[nodeNum][c][i] = nodeDataList[nodeNum][c][i-1];                
        }
    }
    
	nodeDataList[nodeNum][Strength][0] = strength;

	QSettings settings;
    celsius = settings.value("TempInCelsius", true).toBool();

	if (celsius)
	{
		nodeDataList[nodeNum][Temperature][0] = (degrees - 32)/1.8;	// Celsius
	}
	else
		nodeDataList[nodeNum][Temperature][0] = degrees;				// Fahrenheit

	if ((degrees-RANGE) < yAxisMinList[nodeNum][Temperature])
		yAxisMinList[nodeNum][Temperature] = degrees - RANGE;
	if ((degrees+RANGE) < yAxisMaxList[nodeNum][Temperature])
		yAxisMaxList[nodeNum][Temperature] = degrees + RANGE;


	if (dataCountList[nodeNum] == 0)
	{
		baseTimeList[nodeNum] = qCurrTime;

		timeDataList[nodeNum][0] = 0;
	}
	else 
	{
		for ( int i = dataCountList[nodeNum]; i > 0; i-- )
		{       
			if ( i < HISTORY )
				timeDataList[nodeNum][i] = timeDataList[nodeNum][i-1];
		}

		double secondDiff = (qCurrTime.hour()-prevTime.hour())*3600+
			(qCurrTime.minute()-prevTime.minute())*60+
			(qCurrTime.second() - prevTime.second());		

		timeDataList[nodeNum][0] = timeDataList[nodeNum][0]+secondDiff;
	}
	prevTime = qCurrTime;

    if ( dataCountList[nodeNum] < HISTORY )
        dataCountList[nodeNum]++;

    
	if (currentNode == nodeNum)
	{
		setAxisScaleDraw(QwtPlot::xBottom, 
			new TimeScaleDraw(baseTimeList[nodeNum]));

		setAxisScale(QwtPlot::xBottom, 0, HISTORY);
		setAxisLabelRotation(QwtPlot::xBottom, -50.0);
		setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);

		for ( int c = 0; c < NCpuData; c++ )
		{       
			data[c].curve->setRawData(
				timeDataList[nodeNum], nodeDataList[nodeNum][c], dataCountList[nodeNum]);

			
			if (dataCountList[nodeNum] > 0)
			{
				setAxisScale(QwtPlot::xBottom,	
					timeDataList[nodeNum][dataCountList[nodeNum] - 1], timeDataList[nodeNum][0]);
			}
		}
		replot();
	}
}

void SensorPlot::updateCurrentNode(int addr)
{
	currentNode = addr;


	setAxisScaleDraw(QwtPlot::xBottom, 
		new TimeScaleDraw(baseTimeList[currentNode]));

	setAxisScale(QwtPlot::xBottom, 0, HISTORY);
	setAxisLabelRotation(QwtPlot::xBottom, -50.0);
	setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);

	for ( int c = 0; c < NCpuData; c++ )
	{       
		data[c].curve->setRawData(
			timeDataList[currentNode], nodeDataList[currentNode][c], dataCountList[currentNode]);
		
		if (dataCountList[currentNode] > 0)
		{		
			setAxisScale(QwtPlot::xBottom, 
				timeDataList[currentNode][dataCountList[currentNode] - 1], timeDataList[currentNode][0]);
		}
	}
	replot();
}

